@@ -1,4 +1,4 @@
-# test_declarative.py
+# test_schema.py
 # Copyright (C) 2012 the ColanderAlchemy authors and contributors
 # <see AUTHORS file>
 #
@@ -14,11 +14,16 @@ from sqlalchemy import (Column,
                         Integer,
                         BigInteger,
                         TIMESTAMP,
-                        String)
+                        String,
+                        Enum)
 import sqlalchemy
 from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.ext.hybrid import (hybrid_property,
+                                   hybrid_method)
 from sqlalchemy.orm import (mapper,
-                            relationship)
+                            relationship,
+                            synonym,
+                            aliased)
 from sqlalchemy.sql.expression import (text, 
                                        func,
                                        true,
@@ -39,35 +44,39 @@ if sys.version_info[0] == 2 and sys.version_info[1] < 7:
 else:
     import unittest
 
-
+logging.basicConfig(level=logging.WARNING)
 log = logging.getLogger(__name__)
 
 
 class TestsSQLAlchemySchemaNode(unittest.TestCase):
 
     def setUp(self):
-        event.listen(mapper, 'mapper_configured', setup_schema)
+        pass
+
 
     def tearDown(self):
         pass
 
+
     def test_setup_schema(self):
         for cls in [Account, Person, Address]:
-            self.assertEqual(isinstance(cls.__colanderalchemy__,
-                                        SQLAlchemySchemaNode),
-                             True)
+            self.assertIsInstance(cls.__colanderalchemy__,
+                                  SQLAlchemySchemaNode)
+
 
     def test_add_nodes_exceptions(self):
         includes = ('email',)
         excludes = ('email',)
         self.assertRaises(ValueError, SQLAlchemySchemaNode, Account, includes, excludes)
 
+
     def test_default_strategy_for_columns_and_relationships_include_all(self):
         account_schema = SQLAlchemySchemaNode(Account)
         m = sqlalchemy.inspect(Account)
         for attr in m.attrs:
             self.assertIn(attr.key, account_schema)
 
+
     def test_default_strategy_for_included_relationships_schema(self):
         account_schema = SQLAlchemySchemaNode(Account)
         m = sqlalchemy.inspect(Person)
@@ -77,6 +86,7 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         for attr in m.relationships:
             self.assertNotIn(attr.key, account_schema['person'])
 
+
     def test_imperative_includes(self):
         m = sqlalchemy.inspect(Account)
         includes = [attr.key for attr in m.column_attrs]
@@ -95,6 +105,7 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         for attr in m.relationships:
             self.assertIn(attr.key, account_schema)
 
+
     def test_imperative_excludes(self):
         m = sqlalchemy.inspect(Account)
         excludes = [attr.key for attr in m.column_attrs]
@@ -105,6 +116,7 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         for attr in m.relationships:
             self.assertIn(attr.key, account_schema)
 
+
     def test_declarative_excludes(self):
         m = sqlalchemy.inspect(Address)
         address_schema = SQLAlchemySchemaNode(Address)
@@ -114,6 +126,7 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
             if attr.key not in ('city', 'person'):
                 self.assertIn(attr.key, address_schema)
 
+
     def test_imperative_colums_overrides(self):
         overrides = {
             'email': {
@@ -121,18 +134,25 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
             }
         }
         account_schema = SQLAlchemySchemaNode(Account, overrides=overrides)
-        self.assertNotEqual(isinstance(account_schema['email'].typ,
-                                       colander.String),
-                            True)
-        self.assertEqual(isinstance(account_schema['email'].typ,
-                                    colander.Integer),
-                         True)
+        self.assertTrue(
+            isinstance(account_schema['email'].typ, colander.Integer)
+        )
+        overrides = {
+            'email': {
+                'name': 'Name'
+            }
+        }
+        self.assertRaises(ValueError, SQLAlchemySchemaNode, Account, 
+            None, None, overrides)
+
         overrides = {
             'email': {
                 'children': []
             }
         }
-        self.assertRaises(ValueError, SQLAlchemySchemaNode, Account, None, None, overrides)
+        # following shouldn't raise an exception allowing the test to pass
+        SQLAlchemySchemaNode(Account, None, None, overrides)
+
 
     def test_declarative_colums_overrides(self):
         key = SQLAlchemySchemaNode.sqla_info_key
@@ -155,6 +175,7 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         except AttributeError:
             pass
 
+
     def test_imperative_relationships_overrides(self):
         overrides = {
             'person': {
@@ -213,8 +234,8 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         schema = SQLAlchemySchemaNode(Person, overrides=overrides)
         self.assertTrue(isinstance(schema['addresses'].children[0]['id'].typ, colander.String))
 
-    def test_declarative_relationships_overrides(self):
 
+    def test_declarative_relationships_overrides(self):
         key = SQLAlchemySchemaNode.sqla_info_key
         Base = declarative_base()
 
@@ -289,6 +310,7 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         schema = SQLAlchemySchemaNode(Person)
         self.assertTrue(isinstance(schema['addresses'].children[0]['id'].typ, colander.Float))
 
+
     def _prep_schema(self):
         overrides = {
             'person': {
@@ -308,31 +330,79 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         includes = ['email', 'enabled', 'created', 'timeout', 'person']
         schema = SQLAlchemySchemaNode(Account, includes=includes, overrides=overrides)
         #Add a non-SQLAlchemy field
-        schema.add(colander.SchemaNode(colander.String, name='non_sql'))
+        schema.add(colander.SchemaNode(colander.String(), name='non_sql', missing=colander.drop))
         return schema
 
+
     def test_dictify(self):
+        """ Test SQLAlchemySchemaNode.dictify(obj)
+        """
         schema = self._prep_schema()
 
-        args = dict(street='My Street', city='My City')
-        address = Address(**args)
-        kws = dict(name='My Name', surname='My Surname', gender='M', addresses=[address])
-        person = Person(**kws)
-        params = dict(email='mailbox@domain.tld',
+        address_args = dict(street='My Street', city='My City')
+        address = Address(**address_args)
+        
+        person_args = dict(name='My Name', surname='My Surname', 
+                           gender='M', addresses=[address])
+        person = Person(**person_args)
+        
+        account_args = dict(email='mailbox@domain.tld',
                       enabled=True,
                       created=datetime.datetime.now(),
-                      timeout=datetime.time(hour=00, minute=00),
+                      timeout=datetime.time(hour=1, minute=0),
                       person=person)
-        account = Account(**params)
-        dictified = schema.dictify(account)
-        kws['addresses'] = [args]
-        params['person'] = kws
-        self.assertEqual(dictified, params)
-        for key in params:
-            self.assertIn(key, dictified)
-            if key == 'person':
-                for k in kws:
-                    self.assertIn(k, dictified[key])
+        account = Account(**account_args)
+        
+        appstruct = schema.dictify(account)
+        
+        person_args['addresses'] = [address_args]
+        account_args['person'] = person_args
+        self.assertEqual(appstruct, account_args)
+        for account_key in account_args:
+            self.assertIn(account_key, appstruct)
+            if account_key == 'person':
+                for person_key in person_args:
+                    self.assertIn(person_key, appstruct[account_key])
+                    if person_key == 'addresses':
+                        for address_key in address_args:
+                            self.assertIn(address_key, appstruct[account_key][person_key][0])
+        
+        # test that you can serialize this appstruct and you get
+        #  the same result when you deserialize
+        cstruct = schema.serialize(appstruct=appstruct)
+        newappstruct = schema.deserialize(cstruct)
+        self.assertEqual(appstruct, newappstruct)
+
+
+    def test_dictify_with_null(self):
+        """ Test SQLAlchemySchemaNode.dictify(obj) with null values
+        and show that result is a valid appstruct for the given schema
+        """
+        Base = declarative_base()
+        
+        class Sensor(Base):
+            __tablename__ = 'sensor'
+            sensor_id = Column(Integer, primary_key=True)
+            institution_id = Column(Integer, nullable=True)
+            sensor_label = Column(String, nullable=True)
+
+        sensor = Sensor(
+            sensor_id = 3, 
+            institution_id = None, 
+            sensor_label = None,
+        )
+        
+        schema = SQLAlchemySchemaNode(Sensor)
+        appstruct = schema.dictify(sensor)
+        cstruct = schema.serialize(appstruct=appstruct)
+        newappstruct = schema.deserialize(cstruct)
+        newobj = schema.objectify(appstruct)
+        
+        self.assertEqual(appstruct, newappstruct)
+        self.assertEqual(sensor.sensor_id, newobj.sensor_id)
+        self.assertEqual(sensor.institution_id, newobj.institution_id)
+        self.assertEqual(sensor.sensor_label, newobj.sensor_label)
+        
 
     def test_objectify(self):
         """ Test converting a dictionary or data structure into objects.
@@ -357,6 +427,7 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         self.assertEqual(objectified.person.name, 'My Name')
         self.assertFalse(hasattr(objectified, 'foobar'))
 
+
     def test_objectify_context(self):
         """ Test converting a data structure into objects, using a context.
         """
@@ -388,6 +459,7 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         self.assertEqual([node.name for node in schema.children],
                          [node.name for node in cloned.children])
 
+
     def test_schemanode_arguments(self):
         """ Test that any arguments to SchemaNode are accepted.
         """
@@ -399,6 +471,7 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         self.assertEqual(schema.title, 'Dummy')
         self.assertEqual(schema.non_standard, 'Not a Colander arg')
 
+
     def test_read_mapping_configuration(self):
         """ Test using ``__colanderalchemy_config__`` for a mapped class.
         """
@@ -409,16 +482,18 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         self.assertEqual(schema['person'].widget, 'DummyWidget')
         self.assertEqual(schema['person'].title, 'Person Object')
 
+
     def test_missing_mapping_configuration(self):
         """ Test to check ``missing`` is set to an SQLAlchemy-suitable value.
         """
         schema = SQLAlchemySchemaNode(Account)
-        self.assertIsNone(schema['person_id'].missing)
-        self.assertIsNone(schema['person'].missing)
+        self.assertEqual(schema['person_id'].missing, colander.drop)
+        self.assertEqual(schema['person'].missing, colander.drop)
         deserialized = schema.deserialize({'email': 'test@example.com',
                                            'timeout': '09:44:33'})
-        self.assertIsNone(deserialized['person_id'])
-        self.assertIsNone(deserialized['person'])
+        self.assertNotIn('person_id', deserialized)
+        self.assertNotIn('person', deserialized)
+
 
     def test_relationship_mapping_configuration(self):
         """Test to ensure ``missing`` is set to required accordingly.
@@ -437,7 +512,8 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
 
         #Group may have members
         self.assertFalse(schema['members'].required)
-        self.assertEqual(schema['members'].missing, [])
+        self.assertEqual(schema['members'].missing, colander.drop)
+
 
     def test_defaultmissing_primarykey(self):
         """Ensure proper handling of empty values on primary keys
@@ -476,6 +552,7 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         self.assertIn('id', serialized)
         self.assertEqual(serialized['id'], colander.null)
 
+
     def test_default_clause(self):
         """Test proper handling of default and server_default values
         """
@@ -488,7 +565,7 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
             __tablename__ = 'patient'
             # default= is equivalent to ColumnDefault()
             # server_default= is equivalent to DefaultClause()
-            id = Column(BigInteger(unsigned=True), default=text("uuid_short()"), primary_key=True, autoincrement=False)
+            id = Column(BigInteger(), default=text("round(3.14159)"), primary_key=True, autoincrement=False)
             received_timestamp = Column(TIMESTAMP, server_default=func.now(), nullable=False)
             some_number = Column(Integer, DefaultClause('3'), nullable=False)
             scalar_number = Column(Integer, default=3, nullable=False)
@@ -500,7 +577,7 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         Conceivably you should be able to test DefaultClause for a 
         scalar type value and use it as a default/missing in Colander.
         However, the value is interpreted by the backend engine and
-        could be a interpreted by it in an unexpected way.  For this
+        it could be interpreted by it in an unexpected way.  For this
         reason we drop the value and let the backend handle it.
         '''
 
@@ -509,19 +586,18 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         self.assertEqual(schema['received_timestamp'].missing, colander.drop)
         self.assertEqual(schema['some_number'].missing, colander.drop)
         self.assertEqual(schema['scalar_number'].missing, 3)
-        self.assertEqual(schema['pyfunc_test'].missing, 3)
+        self.assertEqual(schema['pyfunc_test'].missing, colander.drop)
         deserialized = schema.deserialize({})
         self.assertIn('scalar_number', deserialized)
         self.assertEqual(deserialized['scalar_number'], 3)
-        self.assertIn('pyfunc_test', deserialized)
-        self.assertEqual(deserialized['pyfunc_test'], 3)
+        self.assertNotIn('pyfunc_test', deserialized)
 
         # from SQLA result into <FORM>; tests default
         self.assertEqual(schema['id'].default, colander.null)
         self.assertEqual(schema['received_timestamp'].default, colander.null)
         self.assertEqual(schema['some_number'].default, colander.null)
         self.assertEqual(schema['scalar_number'].default, 3)
-        self.assertEqual(schema['pyfunc_test'].default, 3)
+        self.assertEqual(schema['pyfunc_test'].default, colander.null)
         serialized = schema.serialize({})
         self.assertIn('id', serialized)
         self.assertEqual(serialized['id'], colander.null)
@@ -532,4 +608,319 @@ class TestsSQLAlchemySchemaNode(unittest.TestCase):
         self.assertIn('scalar_number', serialized)
         self.assertEqual(serialized['scalar_number'], str(3))
         self.assertIn('pyfunc_test', serialized)
-        self.assertEqual(serialized['pyfunc_test'], str(3))
+        self.assertEqual(serialized['pyfunc_test'], colander.null)
+
+
+    def test_unsupported_column_types(self):
+        """
+        Issue #35 - ColanderAlchemy throws when encountering synonyms
+        
+        ColanderAlchemy should ignore synonyms
+        """
+        Base = declarative_base()
+        
+        # example taken from SQLAlchemy docs
+        class MyClass(Base):
+            __tablename__ = 'my_table'
+
+            id = Column(Integer, primary_key=True)
+            job_status = Column(String(50))
+
+            status = synonym("job_status")
+
+        schema = SQLAlchemySchemaNode(MyClass)
+
+        self.assertIn('id', schema)
+        self.assertIn('job_status', schema)
+        self.assertNotIn('status', schema)
+
+    def test_aliased_view(self):
+        """tests aliased view that was broken in 0.3.1
+        """
+        Base = declarative_base()
+
+        class Person(Base):
+            __tablename__ = 'person'
+
+            id = Column(Integer, primary_key=True)
+            parent_id = Column(Integer, ForeignKey('person.id'))
+
+        class ParentView(Base):
+            _parent_table = Person.__table__
+            _child_table = aliased(_parent_table)
+
+            __mapper_args__ = {
+                'primary_key': [_child_table.c.id,],
+                'include_properties': [_child_table.c.id,],
+            }
+
+            __table__ = _parent_table.join(_child_table, _child_table.c.parent_id == _parent_table.c.id)
+
+        schema = SQLAlchemySchemaNode(ParentView)
+        self.assertIn('id', schema)
+
+    def test_hybrid_attributes(self):
+        Base = declarative_base()
+
+        class Interval(Base):
+            __tablename__ = 'interval'
+
+            id = Column(Integer, primary_key=True, info={'id':'id'})
+            start = Column(Integer, nullable=False, info={'start':'start'})
+            end = Column(Integer, nullable=False, info={'end':'end'})
+
+            def __init__(self, start, end):
+                self.start = start
+                self.end = end
+
+            @hybrid_property
+            def length(self):
+                return self.end - self.start
+
+            @hybrid_method
+            def contains(self, point):
+                return (self.start <= point) & (point < self.end)
+
+            @hybrid_method
+            def intersects(self, other):
+                return self.contains(other.start) | self.contains(other.end)
+                
+            @length.setter
+            def length(self, value):
+                self.end = self.start + value
+        
+        
+        schema = SQLAlchemySchemaNode(Interval)
+        
+        self.assertIn('id', schema)
+        self.assertIn('start', schema)
+        self.assertIn('end', schema)
+        self.assertNotIn('length', schema)
+        self.assertNotIn('contains', schema)
+        self.assertNotIn('intersects', schema)
+
+
+    def test_doc_example_deform(self):
+        """
+        Test 'using ColanderAlchemy with Deform' example found
+        in docs/source/deform.rst
+        """
+        Base = declarative_base()
+
+
+        class Phone(Base):
+            __tablename__ = 'phones'
+
+            person_id = Column(Integer, ForeignKey('persons.id'), primary_key=True)
+            number = Column(Unicode(128), primary_key=True)
+            location = Column(Enum(u'home', u'work'))
+
+
+        class Person(Base):
+            __tablename__ = 'persons'
+
+            id = Column(Integer, primary_key=True)
+            name = Column(Unicode(128), nullable=False)
+            surname = Column(Unicode(128), nullable=False)
+            phones = relationship(Phone)
+        
+        
+        schema = SQLAlchemySchemaNode(Person)
+        
+        
+        # because of naming clashes, we need to do this in another function
+        def generate_colander():
+            class Phone(colander.MappingSchema):
+                person_id = colander.SchemaNode(colander.Int())
+                number = colander.SchemaNode(colander.String(),
+                                             validator=colander.Length(0, 128))
+                location = colander.SchemaNode(colander.String(),
+                                               validator=colander.OneOf(['home', 'work']),
+                                               missing=colander.drop)
+
+
+            class Phones(colander.SequenceSchema):
+                phones = Phone(missing=colander.drop)
+
+
+            class Person(colander.MappingSchema):
+                id = colander.SchemaNode(colander.Int(), missing=colander.drop)
+                name = colander.SchemaNode(colander.String(),
+                                           validator=colander.Length(0, 128))
+                surname = colander.SchemaNode(colander.String(),
+                                              validator=colander.Length(0, 128))
+                phones = Phones(missing=colander.drop)
+                
+            
+            return Person()
+        
+        schema2 = generate_colander()
+        
+        self.is_equal_schema(schema, schema2)
+
+
+    def test_doc_example_less_boilerplate(self):
+        """
+        Test 'Examples: Less boilerplate' example
+        found in docs/source/examples.rst
+        """
+        Base = declarative_base()
+
+
+
+        class Phone(Base):
+            __tablename__ = 'phones'
+
+            person_id = Column(Integer, ForeignKey('persons.id'), primary_key=True)
+            number = Column(Unicode(128), primary_key=True)
+            location = Column(Enum(u'home', u'work'))
+
+        class Friend(Base):
+            __tablename__ = 'friends'
+
+            person_id = Column(Integer, ForeignKey('persons.id'), primary_key=True)
+            friend_of = Column(Integer, ForeignKey('persons.id'), primary_key=True)
+            rank = Column(Integer, default=0)
+        
+        class Person(Base):
+            __tablename__ = 'persons'
+
+            id = Column(Integer, primary_key=True)
+            name = Column(Unicode(128), nullable=False)
+            surname = Column(Unicode(128), nullable=False)
+            gender = Column(Enum(u'M', u'F'))
+            age = Column(Integer)
+            phones = relationship(Phone)
+            friends = relationship(Friend, foreign_keys=[Friend.person_id])
+
+        schema = SQLAlchemySchemaNode(Person)
+
+
+        # because of naming clashes, we need to do this in another function
+        def generate_colander():
+            class Friend(colander.MappingSchema):
+                person_id = colander.SchemaNode(colander.Int())
+                friend_of = colander.SchemaNode(colander.Int())
+                rank = colander.SchemaNode(colander.Int(), 
+                                           missing=0, 
+                                           default=0)
+
+
+            class Phone(colander.MappingSchema):
+                person_id = colander.SchemaNode(colander.Int())
+                number = colander.SchemaNode(colander.String(),
+                                             validator=colander.Length(0, 128))
+                location = colander.SchemaNode(colander.String(),
+                                               validator=colander.OneOf(['home', 'work']),
+                                               missing=colander.drop)
+
+
+            class Friends(colander.SequenceSchema):
+                friends = Friend(missing=colander.drop)
+
+
+            class Phones(colander.SequenceSchema):
+                phones = Phone(missing=colander.drop)
+
+
+            class Person(colander.MappingSchema):
+                id = colander.SchemaNode(colander.Int(),
+                                         missing=colander.drop)
+                name = colander.SchemaNode(colander.String(),
+                                           validator=colander.Length(0, 128))
+                surname = colander.SchemaNode(colander.String(),
+                                              validator=colander.Length(0, 128))
+                gender = colander.SchemaNode(colander.String(),
+                                             validator=colander.OneOf(['M', 'F']),
+                                             missing=colander.drop)
+                age = colander.SchemaNode(colander.Int(), 
+                                          missing=colander.drop)
+                phones = Phones(missing=colander.drop)
+                friends = Friends(missing=colander.drop)
+
+
+            return Person()
+
+
+        schema2 = generate_colander()
+        
+        self.is_equal_schema(schema, schema2)
+
+
+    def test_doc_example_Configuring_within_SQLAlchemy(self):
+        """
+        Test 'Configuring within SQLAlchemy models' example
+        found in docs/source/customization.rst
+        """
+        
+        Base = declarative_base()
+
+
+        class Person(Base):
+            __tablename__ = 'person'
+            #Fully customised schema node
+            id = Column(sqlalchemy.Integer,
+                        primary_key=True,
+                        info={'colanderalchemy': {'typ': colander.Float(),
+                                                  'title': 'Person ID',
+                                                  'description': 'The Person identifier.',
+                                                  'widget': 'Empty Widget'}})
+            #Explicitly set as a default field
+            name = Column(sqlalchemy.Unicode(128),
+                          nullable=False,
+                          info={'colanderalchemy': {'default': colander.required}})
+            #Explicitly excluded from resulting schema
+            surname = Column(sqlalchemy.Unicode(128),
+                             nullable=False,
+                             info={'colanderalchemy': {'exclude': True}})
+
+        schema = SQLAlchemySchemaNode(Person)
+
+        # because of naming clashes, we need to do this in another function
+        def generate_colander():
+            class Person(colander.MappingSchema):
+                id = colander.SchemaNode(colander.Float(),
+                                         title='Person ID',
+                                         description='The Person identifier.',
+                                         missing=colander.drop,
+                                         widget='Empty Widget')
+                name = colander.SchemaNode(colander.String(),
+                                           validator=colander.Length(0, 128),
+                                           default=colander.required)
+
+            return Person()
+
+        schema2 = generate_colander()
+        
+        self.is_equal_schema(schema, schema2)
+
+
+    def is_equal_schema(self, schema, schema2, schema_path=None):
+        '''method to compare two colander schema for testing 
+        purposes
+        
+         - does not compare preparer, validator, after_bind, missing_msg
+         - does not do a proper comparison on widgets, but it is
+           sufficient for this test suite
+        '''
+        
+        if schema_path is None:
+            schema_path = []
+        else:
+            schema_path.append(schema.name)
+        self.assertEqual(schema.name, schema2.name)
+        self.assertEqual(type(schema.typ), type(schema2.typ), msg=".".join(schema_path))
+        self.assertEqual(schema.title, schema2.title, msg=".".join(schema_path))
+        self.assertEqual(schema.description, schema2.description, msg=".".join(schema_path))
+        self.assertEqual(schema.missing, schema2.missing, msg=".".join(schema_path))
+        self.assertEqual(schema.default, schema2.default, msg=".".join(schema_path))
+        self.assertEqual(schema.widget, schema2.widget, msg=".".join(schema_path))
+        
+        self.assertEqual(len(schema.children), len(schema2.children))
+        
+        # test children and test that they're in the right order
+        for i, node in enumerate(schema.children):
+            self.assertEqual(node.name, schema2.children[i].name, msg=".".join(schema_path))
+            
+            self.is_equal_schema(node, schema2.children[i], schema_path[:])
+
